@@ -5,29 +5,30 @@ static ID id_txn_close;
 static VALUE
 txn_close_i(VALUE ary)
 {
-    if (rb_respond_to(RARRAY(ary)->ptr[0], id_txn_close)) {
-        rb_funcall(RARRAY(ary)->ptr[0], id_txn_close, 2,
-                   RARRAY(ary)->ptr[1], Qfalse);
+    if (bdb_respond_to(RARRAY_PTR(ary)[0], id_txn_close)) {
+        rb_funcall(RARRAY_PTR(ary)[0], id_txn_close, 2,
+                   RARRAY_PTR(ary)[1], RARRAY_PTR(ary)[2]);
     }
     return Qnil;
 }
 
 static void
-clean_ary(txnst, result)
-    bdb_TXN *txnst;
-    VALUE result;
+clean_ary(bdb_TXN *txnst, VALUE result)
 {
     VALUE *ary, tmp;
     int i, len;
 
-    tmp = rb_assoc_new(Qnil, result);
+    tmp = rb_ary_new2(3);
+    rb_ary_push(tmp, Qnil);
+    rb_ary_push(tmp, result);
+    rb_ary_push(tmp, Qtrue);
     if (txnst->db_ary.ptr) {
         txnst->db_ary.mark = Qtrue;
 	ary = txnst->db_ary.ptr;
 	len = txnst->db_ary.len;
 
 	for (i = 0; i < len; i++) {
-            RARRAY(tmp)->ptr[0] = ary[i];
+            RARRAY_PTR(tmp)[0] = ary[i];
             rb_protect(txn_close_i, tmp, 0);
 	}
         txnst->db_ary.mark = Qfalse;
@@ -35,12 +36,13 @@ clean_ary(txnst, result)
 	txnst->db_ary.total = txnst->db_ary.len = 0;
 	free(ary);
     }
+    RARRAY_PTR(tmp)[2] = Qfalse;
     if (txnst->db_assoc.ptr) {
         txnst->db_assoc.mark = Qtrue;
 	ary = txnst->db_assoc.ptr;
 	len = txnst->db_assoc.len;
 	for (i = 0; i < len; i++) {
-            RARRAY(tmp)->ptr[0] = ary[i];
+            RARRAY_PTR(tmp)[0] = ary[i];
             rb_protect(txn_close_i, tmp, 0);
 	}
         txnst->db_assoc.mark = Qfalse;
@@ -51,8 +53,7 @@ clean_ary(txnst, result)
 }
 
 static VALUE 
-txn_free(txnst)
-    bdb_TXN *txnst;
+txn_free(bdb_TXN *txnst)
 {
     if (txnst->txnid && txnst->parent == NULL) {
 #if BDB_VERSION >= 40000
@@ -70,16 +71,14 @@ txn_free(txnst)
 }
 
 static void
-bdb_txn_free(txnst)
-    bdb_TXN *txnst;
+bdb_txn_free(bdb_TXN *txnst)
 {
-    rb_protect(txn_free, (VALUE)txnst, 0);
+    rb_protect((VALUE (*)(ANYARGS))txn_free, (VALUE)txnst, 0);
     free(txnst);
 }
 
 static void 
-bdb_txn_mark(txnst)
-    bdb_TXN *txnst;
+bdb_txn_mark(bdb_TXN *txnst)
 {
     rb_gc_mark(txnst->marshal);
     rb_gc_mark(txnst->mutex);
@@ -91,10 +90,7 @@ bdb_txn_mark(txnst)
 }
 
 static VALUE
-bdb_txn_assoc(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+bdb_txn_assoc(int argc, VALUE *argv, VALUE obj)
 {
     int i;
     VALUE ary, a;
@@ -107,17 +103,15 @@ bdb_txn_assoc(argc, argv, obj)
 	bdb_ary_push(&txnst->db_assoc, a);
         rb_ary_push(ary, a);
     }
-    switch (RARRAY(ary)->len) {
+    switch (RARRAY_LEN(ary)) {
     case 0: return Qnil;
-    case 1: return RARRAY(ary)->ptr[0];
+    case 1: return RARRAY_PTR(ary)[0];
     default: return ary;
     }
 }
 
 static void
-bdb_txn_close_all(obj, result)
-    VALUE obj;
-    VALUE result;
+bdb_txn_close_all(VALUE obj, VALUE result)
 {
     bdb_TXN *txnst;
     bdb_ENV *envst;
@@ -133,10 +127,7 @@ bdb_txn_close_all(obj, result)
 #define ROLLBACK 3
 
 static VALUE
-bdb_txn_commit(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+bdb_txn_commit(int argc, VALUE *argv, VALUE obj)
 {
     bdb_TXN *txnst;
     VALUE a;
@@ -167,8 +158,7 @@ bdb_txn_commit(argc, argv, obj)
 }
 
 static VALUE
-bdb_txn_abort(obj)
-    VALUE obj;
+bdb_txn_abort(VALUE obj)
 {
     bdb_TXN *txnst;
 
@@ -188,8 +178,7 @@ bdb_txn_abort(obj)
 }
 
 static VALUE
-bdb_txn_unlock(txnv)
-    VALUE txnv;
+bdb_txn_unlock(VALUE txnv)
 {
     bdb_TXN *txnst;
     Data_Get_Struct(txnv, bdb_TXN, txnst);
@@ -200,23 +189,21 @@ bdb_txn_unlock(txnv)
 }
 
 static VALUE
-bdb_catch(val, args, self)
-    VALUE val, args, self;
+bdb_catch(VALUE val, VALUE args, VALUE self)
 {
     rb_yield(args);
     return Qtrue;
 }
 
 static VALUE
-bdb_txn_lock(obj)
-    VALUE obj;
+bdb_txn_lock(VALUE obj)
 {
     VALUE result;
     bdb_TXN *txnst;
     VALUE txnv;
 
     if (TYPE(obj) == T_ARRAY) {
-	txnv = RARRAY(obj)->ptr[0];
+	txnv = RARRAY_PTR(obj)[0];
     }
     else {
 	txnv = obj;
@@ -257,8 +244,7 @@ struct dbtxnopt {
 };
 
 static VALUE
-bdb_txn_i_options(obj, dbstobj)
-    VALUE obj, dbstobj;
+bdb_txn_i_options(VALUE obj, VALUE dbstobj)
 {
     struct dbtxnopt *opt = (struct dbtxnopt *)dbstobj;
     VALUE key, value;
@@ -306,11 +292,7 @@ static VALUE bdb_txn_set_lock_timeout _((VALUE, VALUE));
 #endif
 
 VALUE
-bdb_env_rslbl_begin(origin, argc, argv, obj)
-    VALUE origin;
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+bdb_env_rslbl_begin(VALUE origin, int argc, VALUE *argv, VALUE obj)
 {
     bdb_TXN *txnst, *txnstpar;
     DB_TXN *txn, *txnpar;
@@ -414,8 +396,8 @@ bdb_env_rslbl_begin(origin, argc, argv, obj)
             if (TYPE(b) == T_ARRAY) {
                 long i;
 
-                for (i = 0; i < RARRAY(b)->len; i++) {
-                    rb_ary_push(tmp, RARRAY(b)->ptr[i]);
+                for (i = 0; i < RARRAY_LEN(b); i++) {
+                    rb_ary_push(tmp, RARRAY_PTR(b)[i]);
                 }
             }
             else {
@@ -450,8 +432,7 @@ bdb_env_begin(int argc, VALUE *argv, VALUE obj)
 }
 
 static VALUE
-bdb_txn_id(obj)
-    VALUE obj;
+bdb_txn_id(VALUE obj)
 {
     bdb_TXN *txnst;
     int res;
@@ -467,11 +448,9 @@ bdb_txn_id(obj)
 
 static VALUE
 #if BDB_VERSION >= 30300
-bdb_txn_prepare(obj, txnid)
-    VALUE obj, txnid;
+bdb_txn_prepare(VALUE obj, VALUE txnid)
 #else
-bdb_txn_prepare(obj)
-    VALUE obj;
+bdb_txn_prepare(VALUE obj)
 #endif
 {
     bdb_TXN *txnst;
@@ -493,10 +472,7 @@ bdb_txn_prepare(obj)
 }
 
 static VALUE
-bdb_env_check(argc, argv, obj)
-    int argc;
-    VALUE *argv;
-    VALUE obj;
+bdb_env_check(int argc, VALUE *argv, VALUE obj)
 {
     unsigned int kbyte, min, flag;
     bdb_ENV *envst;
@@ -541,8 +517,7 @@ bdb_env_check(argc, argv, obj)
 #if BDB_VERSION >= 30300
 
 static VALUE
-bdb_env_recover(obj)
-    VALUE obj;
+bdb_env_recover(VALUE obj)
 {
     unsigned int flags;
     long retp;
@@ -578,8 +553,7 @@ bdb_env_recover(obj)
 }
 
 static VALUE
-bdb_txn_discard(obj)
-    VALUE obj;
+bdb_txn_discard(VALUE obj)
 {
     bdb_TXN *txnst;
     int flags;
@@ -599,9 +573,7 @@ bdb_txn_discard(obj)
 #endif
 
 static VALUE
-bdb_env_stat(argc, argv, obj)
-    int argc;
-    VALUE obj, *argv;
+bdb_env_stat(int argc, VALUE *argv, VALUE obj)
 {
     bdb_ENV *envst;
     VALUE a, b;
@@ -675,6 +647,10 @@ bdb_env_stat(argc, argv, obj)
 	    Data_Get_Struct(lsn, struct dblsnst, lsnst);
 	    MEMCPY(lsnst->lsn, &bdb_stat->st_txnarray[i].lsn, DB_LSN, 1);
 	    rb_hash_aset(hash, rb_tainted_str_new2("lsn"), lsn);
+#if BDB_VERSION >= 40416
+	    rb_hash_aset(hash, rb_tainted_str_new2("thread_id"), INT2NUM(bdb_stat->st_txnarray[i].tid));
+	    rb_hash_aset(hash, rb_tainted_str_new2("name"), rb_tainted_str_new2(bdb_stat->st_txnarray[i].name));
+#endif
 	    rb_ary_push(ary, hash);
 	}
     }
@@ -686,8 +662,7 @@ bdb_env_stat(argc, argv, obj)
 #if BDB_VERSION >= 40000
 
 static VALUE
-bdb_txn_set_txn_timeout(obj, a)
-    VALUE obj, a;
+bdb_txn_set_txn_timeout(VALUE obj, VALUE a)
 {
     bdb_TXN *txnst;
 
@@ -699,8 +674,7 @@ bdb_txn_set_txn_timeout(obj, a)
 }
 
 static VALUE
-bdb_txn_set_lock_timeout(obj, a)
-    VALUE obj, a;
+bdb_txn_set_lock_timeout(VALUE obj, VALUE a)
 {
     bdb_TXN *txnst;
 
@@ -712,16 +686,15 @@ bdb_txn_set_lock_timeout(obj, a)
 }
 
 static VALUE
-bdb_txn_set_timeout(obj, a)
-    VALUE obj, a;
+bdb_txn_set_timeout(VALUE obj, VALUE a)
 {
     if (!NIL_P(a)) {
 	if (TYPE(a) == T_ARRAY) {
-	    if (RARRAY(a)->len >= 1 && !NIL_P(RARRAY(a)->ptr[0])) {
-		bdb_txn_set_txn_timeout(obj, RARRAY(a)->ptr[0]);
+	    if (RARRAY_LEN(a) >= 1 && !NIL_P(RARRAY_PTR(a)[0])) {
+		bdb_txn_set_txn_timeout(obj, RARRAY_PTR(a)[0]);
 	    }
-	    if (RARRAY(a)->len == 2 && !NIL_P(RARRAY(a)->ptr[1])) {
-		bdb_txn_set_lock_timeout(obj, RARRAY(a)->ptr[0]);
+	    if (RARRAY_LEN(a) == 2 && !NIL_P(RARRAY_PTR(a)[1])) {
+		bdb_txn_set_lock_timeout(obj, RARRAY_PTR(a)[1]);
 	    }
 	}
 	else {
@@ -734,9 +707,7 @@ bdb_txn_set_timeout(obj, a)
 #if BDB_VERSION >= 40100
 
 static VALUE
-bdb_env_dbremove(argc, argv, obj)
-    int argc;
-    VALUE *argv, obj;
+bdb_env_dbremove(int argc, VALUE *argv, VALUE obj)
 {
     VALUE a, b, c;
     char *file, *database;
@@ -779,9 +750,7 @@ bdb_env_dbremove(argc, argv, obj)
 }
 
 static VALUE
-bdb_env_dbrename(argc, argv, obj)
-    int argc;
-    VALUE *argv, obj;
+bdb_env_dbrename(int argc, VALUE *argv, VALUE obj)
 {
     VALUE a, b, c, d;
     char *file, *database, *newname;
@@ -837,6 +806,31 @@ bdb_env_dbrename(argc, argv, obj)
 
 #endif
 
+#if BDB_VERSION >= 40416
+
+static VALUE
+bdb_txn_set_name(VALUE obj, VALUE a)
+{
+    bdb_TXN *txnst;
+
+    GetTxnDB(obj, txnst);
+    bdb_test_error(txnst->txnid->set_name(txnst->txnid, StringValuePtr(a)));
+    return a;
+}
+
+static VALUE
+bdb_txn_get_name(VALUE obj)
+{
+    bdb_TXN *txnst;
+    const char *name;
+
+    GetTxnDB(obj, txnst);
+    bdb_test_error(txnst->txnid->get_name(txnst->txnid, &name));
+    return rb_tainted_str_new2(name);
+}
+
+#endif
+
 
 void bdb_init_transaction()
 {
@@ -888,6 +882,9 @@ void bdb_init_transaction()
     rb_define_method(bdb_cTxn, "set_timeout", bdb_txn_set_timeout, 1);
     rb_define_method(bdb_cTxn, "set_txn_timeout", bdb_txn_set_txn_timeout, 1);
     rb_define_method(bdb_cTxn, "set_lock_timeout", bdb_txn_set_lock_timeout, 1);
+    rb_define_method(bdb_cTxn, "timeout=", bdb_txn_set_timeout, 1);
+    rb_define_method(bdb_cTxn, "txn_timeout=", bdb_txn_set_txn_timeout, 1);
+    rb_define_method(bdb_cTxn, "lock_timeout=", bdb_txn_set_lock_timeout, 1);
 #if BDB_VERSION >= 40100
     rb_define_method(bdb_cEnv, "dbremove", bdb_env_dbremove, -1);
     rb_define_method(bdb_cTxn, "dbremove", bdb_env_dbremove, -1);
@@ -895,4 +892,8 @@ void bdb_init_transaction()
     rb_define_method(bdb_cTxn, "dbrename", bdb_env_dbrename, -1);
 #endif
 #endif    
+#if BDB_VERSION >= 40416
+    rb_define_method(bdb_cTxn, "name", bdb_txn_get_name, 0);
+    rb_define_method(bdb_cTxn, "name=", bdb_txn_set_name, 1);
+#endif
 }
